import { 
  select, 
  csv, 
  scaleLinear, 
  max, 
  scaleBand, 
  axisLeft,
  axisBottom, 
  format
} from 'd3';

const svg = select('svg');

const width = +svg.attr('width');
const height = +svg.attr('height');
const render = data => {
  const xValue = d => d.population;
  const yValue = d => d.country;
  const margin = { top: 40, bottom: 80, left: 150, right: 50};
  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top - margin.bottom;
  
/* linear scale basically maps your data to pixels or, domain to range, allowing quantitative data to scale to pixels on the screen*/  
  const xScale = scaleLinear()
  	.domain([0, max(data, xValue)])
  	.range([0, innerWidth]);

/* band scales are similar to linear scales in that they have domains and ranges but are used for ordinal attributes */
  const yScale = scaleBand()
  	.domain(data.map(yValue))
  	.range([0, innerHeight])
  	.padding(0.1);
  
  const g = svg.append('g')
  	.attr('transform', `translate(${margin.left}, ${margin.top})`);
  
  // look up d3 number formatting guide
  const xAxisTickFormat = number => 
  	format('.3s')(number)
  		.replace('G', 'B');
  
  const xAxis = axisBottom(xScale)
  	.tickFormat(xAxisTickFormat);

// creating the y axis  
  g.append('g')
    .call(axisLeft(yScale))
  	.selectAll('.domain, .tick line')
  		.remove();
  
  const xAxisG = g.append('g').call(xAxis)
  	.attr('transform', `translate(${0}, ${innerHeight})`);
  
  xAxisG.selectAll('.tick line').remove();
  
  xAxisG.append('text')
  	.attr('class', 'xAxisLabel')
  	.attr('y', 55)
  	.attr('x', innerWidth / 2)
  	.text('Population');

// adding a title to the chart  
  g.append('text')
  	.text('Most Populous Countries')
  	.attr('x', 190)
  	.attr('y', -7);

// adding the bars to the chart  
	g.selectAll('rect').data(data)
  	.enter().append('rect')
  		.attr('y', d => yScale(yValue(d)))
  		.attr('width', d => xScale(xValue(d)))
  		.attr('height', yScale.bandwidth());
};

// bandwidth function is the auto computed width of a single bar

// load data and then for each row convert it to a float and multiply by thousands since data source was given in thousands to get full population

csv('data.csv').then(data => {
  data.forEach(d => {
  	d.population = +d.population * 1000;
    });
  render(data);
});
